#!/bin/sh

# The stub for localization if env. variable LC_TRCOMMAND is not set
lmsg_()
{
	PARGS=
	F_="$1"
	shift

	for i in `echo "$F_" | sed -e's/^[^%]*//g' -e 's/\(%%\)*//g' -e 's/[^%]*%\([[:digit:]]\)\{0,1\}[^%]*/\1 /g' 2>/dev/null`; do
		PARGS="$PARGS \"\$$i\""
	done

	F="`echo "$F_" | sed -e 's/\([^%]\)%\(\(%%\)*\)[[:digit:]]/\1\2%s/g' -e 's/^%\(\(%%\)*\)[[:digit:]]/\1%s/g' 2>/dev/null`"
	eval "printf -- \"\$F\" $PARGS"
}

if [ "use_`( lmsg "%1" lmsg ) 2>/dev/null`" = "use_lmsg" ]; then
	alias _='lmsg rkhunter'
else
	alias _=lmsg_
fi

if [ "$1" = "" -o "$2" = "" -o "$3" = "" -o "$4" = "" -o "$5" = "" ]
  then
    echo "Usage $0 <path/to/rkhunter.conf> <path/to/mirrors.dat> </path/to/dbdir> </path/to/md5>" 
    exit 1
fi 

WGETFOUND=0
CONFFILE=$1
# Mirrors
MIRRORFILE=$2
DBDIR=$3
MD5=$4
LOGFILE=$5

debug()
  {
    echo $1 >> ${LOGFILE}
  }

debug "--------------------------------------------------"
debug "Updater output:"

BINPATHS="/bin /usr/bin /usr/sbin /usr/local/bin /usr/local/sbin /sw/bin"

for I in ${BINPATHS}; do
  J=${I}"/wget"
    if [ -f ${J} ]; then
      WGETFOUND=1
      WGETBINARY=${J}
    fi
done

if [ ${WGETFOUND} -eq 0 ]
  then
    echo "`_ "Wget program required for the Rootkit Hunter database update is not installed.
The Rootkit Hunter database was not updated."`"
    exit 1
fi

# Retrieve file info
FILEINFO=`cat ${CONFFILE} | grep 'UPDATEFILEINFO=' | tr -d 'UPDATEFILEINFO='`

if [ "${FILEINFO}" = "" ]
  then
    echo "Fatal error. Missing line 'UPDATEFILEINFO=' or wrong/non-existing file"
    echo "Please check your configuration file (${CONFFILE})"
    exit 1
fi

checkupdate()
  {
    echo -n "${FILEDESC}: "  
    UPDATEDBURL="${FIRSTMIRROR}/${VERSIONUPDATEURL}"
    LATESTVERSION="`${WGETBINARY} -q -O - ${UPDATEDBURL}`"
    if [ "${LATESTVERSION}" = "" ]
      then
        echo "Failed to retrieve updates."
        echo "Failed to retrieve security knowledge base updates from the update server.
The current knowledge base will be used during security scanning."
        exit 1
    fi
      
    CURRENTVERSION=`cat ${DBDIR}/${FILENAME} | grep '000:version' | cut -d ':' -f3`
    if [ "${CURRENTVERSION}" = "" ]
      then
        CURRENTVERSION=`cat ${DBDIR}/${FILENAME} | grep 'version=' | cut -d '=' -f2`
	if [ "${CURRENTVERSION}" = "" ]
	  then
            echo "Failed to retrieve updates."
            echo "Fatal error: no valid version tag in filename"
	    exit 1
	fi
    fi
    
    if [ "${LATESTVERSION}" = "" ]
      then
        echo "`_ Skipped`"
        echo "`_ "Error: can't obtain valid version tag from downloaded file (or 404 error). Possible outdated mirror."`"
	debug "Tried to fetch ${UPDATEDBURL}"
      else


        if [ ${CURRENTVERSION} -lt ${LATESTVERSION} ]
          then
	    echo "${WHITE}`_ "Update available"`${NORMAL}"

	    # Fetch file
	    GETFILE="${FIRSTMIRROR}/${FILENAME}.gz"
	    TMPFILE="`mktemp /tmp/rkhunter.upd.gz.XXXXXX`" || exit 1
	    
	    if [ ! "`${WGETBINARY} -q -O - ${GETFILE} | gunzip -c > ${TMPFILE}`" ]
	      then	      
	        mv ${TMPFILE} ${DBDIR}/${FILENAME}
		echo "  `_ "Action: Database updated (current version: %1, new version %2)" "${CURRENTVERSION}" "${LATESTVERSION}"`"
	      else
	        echo "`_ "Fatal error: Can't retrieve file: %1" "${GETFILE}"`"
	    fi
	  else
	    if [ ${CURRENTVERSION} -gt ${LATESTVERSION} ]
	      then
	        echo "`_ "Mirror outdated. Skipped"`"
		echo "  `_ "Info (current version: %1, version of mirror: %2)", "${CURRENTVERSION}" "${LATESTVERSION}"`"
	      else
	        echo "`_ "Up to date"`"
	    fi
        fi
    fi
  }


if [ -f ${MIRRORFILE} ]
  then

    MIRRORSVERSION=`cat ${MIRRORFILE} | grep 'version=' | head -n 1`

    # Retrieve first mirror
    FIRSTMIRROR=`cat ${MIRRORFILE} | grep 'mirror=' | head -n 1`
    OTHERMIRRORS=`cat ${MIRRORFILE} | grep -v 'version=' | grep -v ${FIRSTMIRROR}`

    # Clean up files    
    if [ -f ${MIRRORFILE}.new ]; then
        rm -f ${MIRRORFILE}.new
    fi

    touch ${MIRRORFILE}.new
    echo "${MIRRORSVERSION}" > ${MIRRORFILE}.new
    for I in ${OTHERMIRRORS}; do
      echo ${I} >> ${MIRRORFILE}.new
    done;

    echo ${FIRSTMIRROR} >> ${MIRRORFILE}.new
    rm -f ${MIRRORFILE}
    # Use rotated file
    mv ${MIRRORFILE}.new ${MIRRORFILE}
    echo "`_ "Mirrorfile %1 rotated" "${MIRRORFILE}"`"

    FIRSTMIRROR=`echo ${FIRSTMIRROR} | cut -d '=' -f2`
    echo "`_ "Using mirror %1" "${FIRSTMIRROR}"`"

##############################################################################################
    
    LATESTVERSION="unknown"
    FILEDESC="`_ "[DB] Mirror file                      "`"
    FILENAME="mirrors.dat"
    VERSIONUPDATEURL="mirrors.dat.ver"

    checkupdate

###########################
    
    LATESTVERSION="unknown"
    FILEDESC="`_ "[DB] MD5 hashes system binaries       "`"
    FILENAME="defaulthashes.dat"
    VERSIONUPDATEURL="defaulthashes.dat.ver"

    checkupdate

###########################

    LATESTVERSION="unknown"
    FILEDESC="`_ "[DB] Operating System information     "`"
    FILENAME="os.dat"
    VERSIONUPDATEURL="os.dat.ver"

    checkupdate

###########################

    LATESTVERSION="unknown"
    FILEDESC="`_ "[DB] MD5 blacklisted tools/binaries   "`"
    FILENAME="md5blacklist.dat"
    VERSIONUPDATEURL="md5blacklist.dat.ver"

    checkupdate

###########################

    LATESTVERSION="unknown"
    FILEDESC="`_ "[DB] Known good program versions      "`"
    FILENAME="programs_good.dat"
    VERSIONUPDATEURL="programs_good.dat.ver"

    checkupdate

###########################

    LATESTVERSION="unknown"
    FILEDESC="`_ "[DB] Known bad program versions       "`"
    FILENAME="programs_bad.dat"
    VERSIONUPDATEURL="programs_bad.dat.ver"

    checkupdate

##############################################################################################

    echo "" ; echo ""; echo ""

  else

    echo "`_ "Fatal error: %1 does not exist" "${MIRRORFILE}"`"
    exit 1

fi
